# This file was auto-generated by Fern from our API Definition.

import typing

from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.precondition import Precondition
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawSegmentsClient, RawSegmentsClient
from .types.segment_field import SegmentField
from .types.segment_response import SegmentResponse
from .types.segment_status import SegmentStatus
from .types.segments_search_response import SegmentsSearchResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SegmentsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSegmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSegmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSegmentsClient
        """
        return self._raw_client

    def search(
        self,
        *,
        sort: typing.Optional[SegmentField] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentsSearchResponse:
        """
        Parameters
        ----------
        sort : typing.Optional[SegmentField]
            The field to sort by, defaults to created timestamp

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentsSearchResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.segments.search()
        """
        _response = self._raw_client.search(
            sort=sort, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    def create_or_update(
        self,
        *,
        segment_id: EntityIdBase,
        name: str,
        precondition: Precondition,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentResponse:
        """
        Update a segment or create it if it doesn't exist.

        Parameters
        ----------
        segment_id : EntityIdBase
            ID that uniquely identifies this segment

        name : str
            The name of the segment.

        precondition : Precondition
            The precondition that must be met for a conversation message to be included in the segment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentResponse

        Examples
        --------
        from mavenagi import MavenAGI
        from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.segments.create_or_update(
            segment_id=EntityIdBase(
                reference_id="admin-users",
            ),
            name="Admin users",
            precondition=Precondition_Group(
                operator="AND",
                preconditions=[
                    Precondition_User(
                        key="userKey",
                    ),
                    Precondition_User(
                        key="userKey2",
                    ),
                ],
            ),
        )
        """
        _response = self._raw_client.create_or_update(
            segment_id=segment_id, name=name, precondition=precondition, request_options=request_options
        )
        return _response.data

    def get(
        self,
        segment_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentResponse:
        """
        Get a segment by its supplied ID

        Parameters
        ----------
        segment_reference_id : str
            The reference ID of the segment to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the segment to get. If not provided, the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.segments.get(
            segment_reference_id="admin-users",
        )
        """
        _response = self._raw_client.get(segment_reference_id, app_id=app_id, request_options=request_options)
        return _response.data

    def patch(
        self,
        segment_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        status: typing.Optional[SegmentStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentResponse:
        """
        Update mutable segment fields

        The `appId` field can be provided to update a segment owned by a different app.
        All other fields will overwrite the existing value on the segment only if provided.

        Parameters
        ----------
        segment_reference_id : str
            The reference ID of the segment to update. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the segment to update. If not provided, the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the segment.

        precondition : typing.Optional[Precondition]
            The precondition that must be met for a conversation message to be included in the segment.

        status : typing.Optional[SegmentStatus]
            The status of the segment. Segments can only be deactivated if they are not set on any actions or active knowledge bases.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentResponse
            The updated segment

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.segments.patch(
            segment_reference_id="segmentReferenceId",
        )
        """
        _response = self._raw_client.patch(
            segment_reference_id,
            app_id=app_id,
            name=name,
            precondition=precondition,
            status=status,
            request_options=request_options,
        )
        return _response.data


class AsyncSegmentsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSegmentsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSegmentsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSegmentsClient
        """
        return self._raw_client

    async def search(
        self,
        *,
        sort: typing.Optional[SegmentField] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentsSearchResponse:
        """
        Parameters
        ----------
        sort : typing.Optional[SegmentField]
            The field to sort by, defaults to created timestamp

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentsSearchResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.segments.search()


        asyncio.run(main())
        """
        _response = await self._raw_client.search(
            sort=sort, page=page, size=size, sort_desc=sort_desc, request_options=request_options
        )
        return _response.data

    async def create_or_update(
        self,
        *,
        segment_id: EntityIdBase,
        name: str,
        precondition: Precondition,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentResponse:
        """
        Update a segment or create it if it doesn't exist.

        Parameters
        ----------
        segment_id : EntityIdBase
            ID that uniquely identifies this segment

        name : str
            The name of the segment.

        precondition : Precondition
            The precondition that must be met for a conversation message to be included in the segment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI
        from mavenagi.commons import EntityIdBase, Precondition_Group, Precondition_User

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.segments.create_or_update(
                segment_id=EntityIdBase(
                    reference_id="admin-users",
                ),
                name="Admin users",
                precondition=Precondition_Group(
                    operator="AND",
                    preconditions=[
                        Precondition_User(
                            key="userKey",
                        ),
                        Precondition_User(
                            key="userKey2",
                        ),
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_or_update(
            segment_id=segment_id, name=name, precondition=precondition, request_options=request_options
        )
        return _response.data

    async def get(
        self,
        segment_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentResponse:
        """
        Get a segment by its supplied ID

        Parameters
        ----------
        segment_reference_id : str
            The reference ID of the segment to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the segment to get. If not provided, the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.segments.get(
                segment_reference_id="admin-users",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(segment_reference_id, app_id=app_id, request_options=request_options)
        return _response.data

    async def patch(
        self,
        segment_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        status: typing.Optional[SegmentStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SegmentResponse:
        """
        Update mutable segment fields

        The `appId` field can be provided to update a segment owned by a different app.
        All other fields will overwrite the existing value on the segment only if provided.

        Parameters
        ----------
        segment_reference_id : str
            The reference ID of the segment to update. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the segment to update. If not provided, the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the segment.

        precondition : typing.Optional[Precondition]
            The precondition that must be met for a conversation message to be included in the segment.

        status : typing.Optional[SegmentStatus]
            The status of the segment. Segments can only be deactivated if they are not set on any actions or active knowledge bases.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SegmentResponse
            The updated segment

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.segments.patch(
                segment_reference_id="segmentReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.patch(
            segment_reference_id,
            app_id=app_id,
            name=name,
            precondition=precondition,
            status=status,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .segment_base import SegmentBase
from .segment_field import SegmentField
from .segment_patch_request import SegmentPatchRequest
from .segment_request import SegmentRequest
from .segment_response import SegmentResponse
from .segment_status import SegmentStatus
from .segments_search_request import SegmentsSearchRequest
from .segments_search_response import SegmentsSearchResponse

__all__ = [
    "SegmentBase",
    "SegmentField",
    "SegmentPatchRequest",
    "SegmentRequest",
    "SegmentResponse",
    "SegmentStatus",
    "SegmentsSearchRequest",
    "SegmentsSearchResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .context_info import ContextInfo
from .scoped_entity import ScopedEntity
from .session_info import SessionInfo
from .source_info import SourceInfo


class EventBaseNoId(UniversalBaseModel):
    timestamp: typing.Optional[dt.datetime] = None
    references: typing.Optional[typing.List[ScopedEntity]] = None
    source_info: typing_extensions.Annotated[typing.Optional[SourceInfo], FieldMetadata(alias="sourceInfo")] = None
    session_info: typing_extensions.Annotated[typing.Optional[SessionInfo], FieldMetadata(alias="sessionInfo")] = None
    context_info: typing_extensions.Annotated[typing.Optional[ContextInfo], FieldMetadata(alias="contextInfo")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

AgentCapabilityField = typing.Union[
    typing.Literal["CreatedAt", "Enabled", "Name", "Type", "UserInteractionRequired"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...users.types.agent_user_filter import AgentUserFilter


class AgentUserAnalyticsRequest(UniversalBaseModel):
    agent_user_filter: typing_extensions.Annotated[
        typing.Optional[AgentUserFilter], FieldMetadata(alias="agentUserFilter")
    ] = pydantic.Field(default=None)
    """
    Optional filter applied to refine the agent user data before processing.
    """

    timezone: typing.Optional[str] = pydantic.Field(default=None)
    """
    IANA timezone identifier (e.g., "America/Los_Angeles").
    When provided, time-based groupings (e.g., DAY) and date filters are evaluated in this timezone;
    otherwise UTC is used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .action_capability import ActionCapability
from .agent_capability import AgentCapability, AgentCapability_Action, AgentCapability_Trigger
from .agent_capability_base import AgentCapabilityBase
from .agent_capability_field import AgentCapabilityField
from .agent_capability_list_request import AgentCapabilityListRequest
from .agent_capability_type import AgentCapabilityType
from .execute_capability_request import ExecuteCapabilityRequest
from .execute_capability_response import ExecuteCapabilityResponse
from .list_agent_capabilities_response import ListAgentCapabilitiesResponse
from .patch_agent_capability_request import PatchAgentCapabilityRequest
from .trigger_capability import TriggerCapability
from .trigger_type import TriggerType

__all__ = [
    "ActionCapability",
    "AgentCapability",
    "AgentCapabilityBase",
    "AgentCapabilityField",
    "AgentCapabilityListRequest",
    "AgentCapabilityType",
    "AgentCapability_Action",
    "AgentCapability_Trigger",
    "ExecuteCapabilityRequest",
    "ExecuteCapabilityResponse",
    "ListAgentCapabilitiesResponse",
    "PatchAgentCapabilityRequest",
    "TriggerCapability",
    "TriggerType",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class UpdateMetadataRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mavenagi.conversation import UpdateMetadataRequest

    UpdateMetadataRequest(
        app_id="conversation-owning-app",
        values={"key": "newValue"},
    )
    """

    app_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="appId")] = pydantic.Field(
        default=None
    )
    """
    The App ID of the conversation to modify metadata for. If not provided the ID of the calling app will be used.
    """

    values: typing.Dict[str, str] = pydantic.Field()
    """
    The metadata values to add to the conversation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...commons.types.entity_id_without_agent import EntityIdWithoutAgent
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .base_knowledge_document import BaseKnowledgeDocument
from .knowledge_document_content_type import KnowledgeDocumentContentType


class KnowledgeDocumentRequest(BaseKnowledgeDocument):
    """
    Examples
    --------
    from mavenagi.commons import EntityIdBase, EntityIdWithoutAgent
    from mavenagi.knowledge import KnowledgeDocumentRequest

    KnowledgeDocumentRequest(
        knowledge_document_id=EntityIdBase(
            reference_id="getting-started",
        ),
        version_id=EntityIdWithoutAgent(
            type="KNOWLEDGE_BASE_VERSION",
            reference_id="versionId",
            app_id="maven",
        ),
        content_type="MARKDOWN",
        content="## Getting started\\nThis is a getting started guide for the help center.",
        title="Getting started",
        metadata={"category": "getting-started"},
    )
    """

    knowledge_document_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="knowledgeDocumentId")] = (
        pydantic.Field()
    )
    """
    ID that uniquely identifies this knowledge document within its knowledge base
    """

    version_id: typing_extensions.Annotated[typing.Optional[EntityIdWithoutAgent], FieldMetadata(alias="versionId")] = (
        pydantic.Field(default=None)
    )
    """
    ID that uniquely identifies which knowledge base version to create the document in. If not provided will use the most recent version of the knowledge base.
    """

    content_type: typing_extensions.Annotated[KnowledgeDocumentContentType, FieldMetadata(alias="contentType")]
    content: str = pydantic.Field()
    """
    The content of the document. Not shown directly to users. May be provided in HTML or markdown. HTML will be converted to markdown automatically. Images are not currently supported and will be ignored.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata for the knowledge document.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class Page(UniversalBaseModel):
    number: int = pydantic.Field()
    """
    The page being returned, starts at 0
    """

    size: int = pydantic.Field()
    """
    The number of elements in this page
    """

    total_elements: typing_extensions.Annotated[int, FieldMetadata(alias="totalElements")] = pydantic.Field()
    """
    The total number of elements in the collection
    """

    total_pages: typing_extensions.Annotated[int, FieldMetadata(alias="totalPages")] = pydantic.Field()
    """
    The total number of pages in the collection
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .entity_id_without_agent import EntityIdWithoutAgent


class BotLogicKnowledgeDetail(UniversalBaseModel):
    knowledge_base_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="knowledgeBaseId")]
    document_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="documentId")]
    document_name: typing_extensions.Annotated[str, FieldMetadata(alias="documentName")]
    document_excerpt: typing_extensions.Annotated[str, FieldMetadata(alias="documentExcerpt")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .agent_user import AgentUser
from .agent_user_field import AgentUserField
from .agent_user_filter import AgentUserFilter
from .agent_user_search_request import AgentUserSearchRequest
from .agent_user_search_response import AgentUserSearchResponse
from .app_user_summary import AppUserSummary

__all__ = [
    "AgentUser",
    "AgentUserField",
    "AgentUserFilter",
    "AgentUserSearchRequest",
    "AgentUserSearchResponse",
    "AppUserSummary",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    SegmentBase,
    SegmentField,
    SegmentPatchRequest,
    SegmentRequest,
    SegmentResponse,
    SegmentStatus,
    SegmentsSearchRequest,
    SegmentsSearchResponse,
)

__all__ = [
    "SegmentBase",
    "SegmentField",
    "SegmentPatchRequest",
    "SegmentRequest",
    "SegmentResponse",
    "SegmentStatus",
    "SegmentsSearchRequest",
    "SegmentsSearchResponse",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...commons.types.error_message import ErrorMessage
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ObjectStreamResponse_Text(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["text"], FieldMetadata(alias="eventType")] = "text"
    contents: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ObjectStreamResponse_Start(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["start"], FieldMetadata(alias="eventType")] = "start"
    conversation_message_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="conversationMessageId")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ObjectStreamResponse_End(UniversalBaseModel):
    event_type: typing_extensions.Annotated[typing.Literal["end"], FieldMetadata(alias="eventType")] = "end"
    error: typing.Optional[ErrorMessage] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ObjectStreamResponse = typing.Union[ObjectStreamResponse_Text, ObjectStreamResponse_Start, ObjectStreamResponse_End]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .inbox_item_fix_base import InboxItemFixBase


class InboxItemFixAddDocument(InboxItemFixBase):
    suggested_text_title: typing_extensions.Annotated[str, FieldMetadata(alias="suggestedTextTitle")] = pydantic.Field()
    """
    Suggested document title if the fix type is ADD_DOCUMENT.
    """

    suggested_text: typing_extensions.Annotated[str, FieldMetadata(alias="suggestedText")] = pydantic.Field()
    """
    Suggested document text if the fix type is ADD_DOCUMENT.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class AgentUserFilter(UniversalBaseModel):
    search: typing.Optional[str] = pydantic.Field(default=None)
    """
    Full-text search query for matching agent users by content.
    When you search with this parameter, you're performing a full-text search across the user identifiers.
    """

    identifiers: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter by identifiers
    """

    is_anonymous: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="isAnonymous")] = (
        pydantic.Field(default=None)
    )
    """
    Filter by anonymous users. When true, only anonymous users are returned.
    When false, only non-anonymous users are returned. An anonymous user is one without any identifiers or name data.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.base_paginated_request import BasePaginatedRequest
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .agent_capability_field import AgentCapabilityField
from .agent_capability_type import AgentCapabilityType


class AgentCapabilityListRequest(BasePaginatedRequest):
    capability_type: typing_extensions.Annotated[
        typing.Optional[AgentCapabilityType], FieldMetadata(alias="capabilityType")
    ] = None
    pinned: typing.Optional[bool] = None
    enabled: typing.Optional[bool] = None
    integration_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="integrationIds")
    ] = None
    description: typing.Optional[str] = None
    user_interaction_required: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="userInteractionRequired")
    ] = None
    sort_id: typing_extensions.Annotated[typing.Optional[AgentCapabilityField], FieldMetadata(alias="sortId")] = (
        pydantic.Field(default=None)
    )
    """
    The field to sort by, defaults to created timestamp
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .document_information import DocumentInformation
from .entity_id import EntityId


class InboxItemFix_AddDocument(UniversalBaseModel):
    type: typing.Literal["addDocument"] = "addDocument"
    suggested_text_title: typing_extensions.Annotated[str, FieldMetadata(alias="suggestedTextTitle")]
    suggested_text: typing_extensions.Annotated[str, FieldMetadata(alias="suggestedText")]
    id: EntityId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class InboxItemFix_DeactivateDocument(UniversalBaseModel):
    type: typing.Literal["deactivateDocument"] = "deactivateDocument"
    document_information: typing_extensions.Annotated[DocumentInformation, FieldMetadata(alias="documentInformation")]
    id: EntityId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


InboxItemFix = typing.Union[InboxItemFix_AddDocument, InboxItemFix_DeactivateDocument]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ...commons.types.page import Page
from ...core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from .agent_capability import AgentCapability


class ListAgentCapabilitiesResponse(Page):
    count: int
    data: typing.List[AgentCapability]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


from ...commons.types.precondition_group import PreconditionGroup  # noqa: E402, F401, I001

update_forward_refs(ListAgentCapabilitiesResponse)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.entity_id import EntityId
from ..commons.types.entity_id_base import EntityIdBase
from ..commons.types.entity_id_without_agent import EntityIdWithoutAgent
from ..commons.types.error_message import ErrorMessage
from ..commons.types.llm_inclusion_status import LlmInclusionStatus
from ..commons.types.precondition import Precondition
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.knowledge_base_field import KnowledgeBaseField
from .types.knowledge_base_filter import KnowledgeBaseFilter
from .types.knowledge_base_refresh_frequency import KnowledgeBaseRefreshFrequency
from .types.knowledge_base_response import KnowledgeBaseResponse
from .types.knowledge_base_version import KnowledgeBaseVersion
from .types.knowledge_base_version_finalize_status import KnowledgeBaseVersionFinalizeStatus
from .types.knowledge_base_version_type import KnowledgeBaseVersionType
from .types.knowledge_base_versions_list_response import KnowledgeBaseVersionsListResponse
from .types.knowledge_bases_response import KnowledgeBasesResponse
from .types.knowledge_document_content_type import KnowledgeDocumentContentType
from .types.knowledge_document_field import KnowledgeDocumentField
from .types.knowledge_document_filter import KnowledgeDocumentFilter
from .types.knowledge_document_response import KnowledgeDocumentResponse
from .types.knowledge_documents_response import KnowledgeDocumentsResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawKnowledgeClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_knowledge_bases(
        self,
        *,
        sort: typing.Optional[KnowledgeBaseField] = OMIT,
        filter: typing.Optional[KnowledgeBaseFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBasesResponse]:
        """
        Search knowledge bases

        Parameters
        ----------
        sort : typing.Optional[KnowledgeBaseField]

        filter : typing.Optional[KnowledgeBaseFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBasesResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/knowledge/search",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=KnowledgeBaseFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBasesResponse,
                    parse_obj_as(
                        type_=KnowledgeBasesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_or_update_knowledge_base(
        self,
        *,
        knowledge_base_id: EntityIdBase,
        name: str,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseResponse]:
        """
        Update a knowledge base or create it if it doesn't exist.

        Parameters
        ----------
        knowledge_base_id : EntityIdBase
            ID that uniquely identifies this knowledge base

        name : str
            The name of the knowledge base

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata for the knowledge base.

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for knowledge base be relevant to a conversation. Can be used to restrict knowledge bases to certain types of users.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/knowledge",
            method="PUT",
            json={
                "knowledgeBaseId": convert_and_respect_annotation_metadata(
                    object_=knowledge_base_id, annotation=EntityIdBase, direction="write"
                ),
                "metadata": metadata,
                "name": name,
                "precondition": convert_and_respect_annotation_metadata(
                    object_=precondition, annotation=Precondition, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_knowledge_base(
        self,
        knowledge_base_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseResponse]:
        """
        Get an existing knowledge base by its supplied ID

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the knowledge base to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch_knowledge_base(
        self,
        knowledge_base_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Set[str]] = OMIT,
        llm_inclusion_status: typing.Optional[LlmInclusionStatus] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        segment_id: typing.Optional[EntityId] = OMIT,
        refresh_frequency: typing.Optional[KnowledgeBaseRefreshFrequency] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseResponse]:
        """
        Update mutable knowledge base fields

        The `appId` field can be provided to update a knowledge base owned by a different app.
        All other fields will overwrite the existing value on the knowledge base only if provided.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to patch.

        app_id : typing.Optional[str]
            The App ID of the knowledge base to patch. If not provided the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the knowledge base.

        tags : typing.Optional[typing.Set[str]]
            The tags of the knowledge base.

        llm_inclusion_status : typing.Optional[LlmInclusionStatus]
            Determines whether documents in the knowledge base are sent to the LLM as part of a conversation. Note that at this time knowledge bases can not be set to `ALWAYS`.

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for a knowledge base to be relevant to a conversation. Can be used to restrict knowledge bases to certain types of users. A null value will remove the precondition from the knowledge base, it will be available on all conversations.

        segment_id : typing.Optional[EntityId]
            The ID of the segment that must be matched for the knowledge base to be relevant to a conversation.
            A null value will remove the segment from the knowledge base, it will be available on all conversations.

            Segments are replacing inline preconditions - a knowledge base may not have both an inline precondition and a segment.
            Inline precondition support will be removed in a future release.

        refresh_frequency : typing.Optional[KnowledgeBaseRefreshFrequency]
            How often the knowledge base should be refreshed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}",
            method="PATCH",
            json={
                "appId": app_id,
                "name": name,
                "tags": tags,
                "llmInclusionStatus": llm_inclusion_status,
                "precondition": convert_and_respect_annotation_metadata(
                    object_=precondition, annotation=typing.Optional[Precondition], direction="write"
                ),
                "segmentId": convert_and_respect_annotation_metadata(
                    object_=segment_id, annotation=typing.Optional[EntityId], direction="write"
                ),
                "refreshFrequency": refresh_frequency,
            },
            headers={
                "content-type": "application/merge-patch+json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_knowledge_base_version(
        self,
        knowledge_base_reference_id: str,
        *,
        type: KnowledgeBaseVersionType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseVersion]:
        """
        Create a new knowledge base version.

        If an existing version is in progress, then that version will be finalized in an error state.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to create a version for. All other entity ID fields are inferred from the request.

        type : KnowledgeBaseVersionType
            Indicates whether the completed version constitutes a full or partial refresh of the knowledge base. Deleting and updating documents is only supported for partial refreshes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseVersion]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/version",
            method="POST",
            json={
                "type": type,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseVersion,
                    parse_obj_as(
                        type_=KnowledgeBaseVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def finalize_knowledge_base_version(
        self,
        knowledge_base_reference_id: str,
        *,
        version_id: typing.Optional[EntityIdWithoutAgent] = OMIT,
        status: typing.Optional[KnowledgeBaseVersionFinalizeStatus] = OMIT,
        error_message: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseVersion]:
        """
        Finalize the latest knowledge base version. Required to indicate the version is complete. Will throw an exception if the latest version is not in progress.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to finalize a version for. All other entity ID fields are inferred from the request.

        version_id : typing.Optional[EntityIdWithoutAgent]
            ID that uniquely identifies which knowledge base version to finalize. If not provided will use the most recent version of the knowledge base.

        status : typing.Optional[KnowledgeBaseVersionFinalizeStatus]
            Whether the knowledge base version processing was successful or not.

        error_message : typing.Optional[str]
            A user-facing error message that provides more details about a version failure.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseVersion]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/version/finalize",
            method="POST",
            json={
                "versionId": convert_and_respect_annotation_metadata(
                    object_=version_id, annotation=EntityIdWithoutAgent, direction="write"
                ),
                "status": status,
                "errorMessage": error_message,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseVersion,
                    parse_obj_as(
                        type_=KnowledgeBaseVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_knowledge_base_versions(
        self,
        knowledge_base_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseVersionsListResponse]:
        """
        List all active versions for a knowledge base. Returns the most recent versions first.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to list versions for. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the knowledge base. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseVersionsListResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/versions",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseVersionsListResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseVersionsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def search_knowledge_documents(
        self,
        *,
        sort: typing.Optional[KnowledgeDocumentField] = OMIT,
        filter: typing.Optional[KnowledgeDocumentFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeDocumentsResponse]:
        """
        Search knowledge documents

        Parameters
        ----------
        sort : typing.Optional[KnowledgeDocumentField]

        filter : typing.Optional[KnowledgeDocumentFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeDocumentsResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/knowledge/documents/search",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=KnowledgeDocumentFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeDocumentsResponse,
                    parse_obj_as(
                        type_=KnowledgeDocumentsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_knowledge_document(
        self,
        knowledge_base_reference_id: str,
        *,
        knowledge_document_id: EntityIdBase,
        content_type: KnowledgeDocumentContentType,
        content: str,
        title: str,
        version_id: typing.Optional[EntityIdWithoutAgent] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        url: typing.Optional[str] = OMIT,
        language: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        author: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeDocumentResponse]:
        """
        Create or update a knowledge document. Requires an existing knowledge base with an in progress version.
        Will throw an exception if the latest version is not in progress.

        <Tip>
        This API maintains document version history. If for the same reference ID none of the `title`, `text`, `sourceUrl`, `metadata` fields
        have changed, a new document version will not be created. The existing version will be reused.
        </Tip>

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to create a document for. All other entity ID fields are inferred from the request.

        knowledge_document_id : EntityIdBase
            ID that uniquely identifies this knowledge document within its knowledge base

        content_type : KnowledgeDocumentContentType

        content : str
            The content of the document. Not shown directly to users. May be provided in HTML or markdown. HTML will be converted to markdown automatically. Images are not currently supported and will be ignored.

        title : str
            The title of the document. Will be shown as part of answers.

        version_id : typing.Optional[EntityIdWithoutAgent]
            ID that uniquely identifies which knowledge base version to create the document in. If not provided will use the most recent version of the knowledge base.

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata for the knowledge document.

        url : typing.Optional[str]
            The URL of the document. Should be visible to end users. Will be shown as part of answers. Not used for crawling.

        language : typing.Optional[str]
            The document language. Must be a valid ISO 639-1 language code.

        created_at : typing.Optional[dt.datetime]
            The time at which this document was created.

        updated_at : typing.Optional[dt.datetime]
            The time at which this document was last modified.

        author : typing.Optional[str]
            The name of the author who created this document.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeDocumentResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/document",
            method="POST",
            json={
                "knowledgeDocumentId": convert_and_respect_annotation_metadata(
                    object_=knowledge_document_id, annotation=EntityIdBase, direction="write"
                ),
                "versionId": convert_and_respect_annotation_metadata(
                    object_=version_id, annotation=EntityIdWithoutAgent, direction="write"
                ),
                "contentType": content_type,
                "content": content,
                "metadata": metadata,
                "title": title,
                "url": url,
                "language": language,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "author": author,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeDocumentResponse,
                    parse_obj_as(
                        type_=KnowledgeDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_knowledge_document(
        self,
        knowledge_base_reference_id: str,
        knowledge_document_reference_id: str,
        *,
        version_id: EntityIdWithoutAgent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Delete knowledge document from a specific version.
        Requires an existing knowledge base with an in progress version of type PARTIAL. Will throw an exception if the version is not in progress.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base that contains the document to delete. All other entity ID fields are inferred from the request

        knowledge_document_reference_id : str
            The reference ID of the knowledge document to delete. All other entity ID fields are inferred from the request.

        version_id : EntityIdWithoutAgent
            ID that uniquely identifies which knowledge base version to delete the document from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/{jsonable_encoder(knowledge_document_reference_id)}/document",
            method="DELETE",
            json={
                "versionId": convert_and_respect_annotation_metadata(
                    object_=version_id, annotation=EntityIdWithoutAgent, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_knowledge_document(
        self,
        knowledge_base_version_reference_id: str,
        knowledge_document_reference_id: str,
        *,
        knowledge_base_version_app_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeDocumentResponse]:
        """
        Get a knowledge document by its supplied version and document IDs. Response includes document content in markdown format.

        Parameters
        ----------
        knowledge_base_version_reference_id : str
            The reference ID of the knowledge base version that contains the document. All other entity ID fields are inferred from the request.

        knowledge_document_reference_id : str
            The reference ID of the knowledge document to get. All other entity ID fields are inferred from the request.

        knowledge_base_version_app_id : str
            The App ID of the knowledge base version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeDocumentResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/knowledge/versions/{jsonable_encoder(knowledge_base_version_reference_id)}/documents/{jsonable_encoder(knowledge_document_reference_id)}",
            method="GET",
            params={
                "knowledgeBaseVersionAppId": knowledge_base_version_app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeDocumentResponse,
                    parse_obj_as(
                        type_=KnowledgeDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawKnowledgeClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_knowledge_bases(
        self,
        *,
        sort: typing.Optional[KnowledgeBaseField] = OMIT,
        filter: typing.Optional[KnowledgeBaseFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBasesResponse]:
        """
        Search knowledge bases

        Parameters
        ----------
        sort : typing.Optional[KnowledgeBaseField]

        filter : typing.Optional[KnowledgeBaseFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBasesResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/knowledge/search",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=KnowledgeBaseFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBasesResponse,
                    parse_obj_as(
                        type_=KnowledgeBasesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_or_update_knowledge_base(
        self,
        *,
        knowledge_base_id: EntityIdBase,
        name: str,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseResponse]:
        """
        Update a knowledge base or create it if it doesn't exist.

        Parameters
        ----------
        knowledge_base_id : EntityIdBase
            ID that uniquely identifies this knowledge base

        name : str
            The name of the knowledge base

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata for the knowledge base.

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for knowledge base be relevant to a conversation. Can be used to restrict knowledge bases to certain types of users.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/knowledge",
            method="PUT",
            json={
                "knowledgeBaseId": convert_and_respect_annotation_metadata(
                    object_=knowledge_base_id, annotation=EntityIdBase, direction="write"
                ),
                "metadata": metadata,
                "name": name,
                "precondition": convert_and_respect_annotation_metadata(
                    object_=precondition, annotation=Precondition, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_knowledge_base(
        self,
        knowledge_base_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseResponse]:
        """
        Get an existing knowledge base by its supplied ID

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to get. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the knowledge base to get. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch_knowledge_base(
        self,
        knowledge_base_reference_id: str,
        *,
        app_id: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.Set[str]] = OMIT,
        llm_inclusion_status: typing.Optional[LlmInclusionStatus] = OMIT,
        precondition: typing.Optional[Precondition] = OMIT,
        segment_id: typing.Optional[EntityId] = OMIT,
        refresh_frequency: typing.Optional[KnowledgeBaseRefreshFrequency] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseResponse]:
        """
        Update mutable knowledge base fields

        The `appId` field can be provided to update a knowledge base owned by a different app.
        All other fields will overwrite the existing value on the knowledge base only if provided.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to patch.

        app_id : typing.Optional[str]
            The App ID of the knowledge base to patch. If not provided the ID of the calling app will be used.

        name : typing.Optional[str]
            The name of the knowledge base.

        tags : typing.Optional[typing.Set[str]]
            The tags of the knowledge base.

        llm_inclusion_status : typing.Optional[LlmInclusionStatus]
            Determines whether documents in the knowledge base are sent to the LLM as part of a conversation. Note that at this time knowledge bases can not be set to `ALWAYS`.

        precondition : typing.Optional[Precondition]
            The preconditions that must be met for a knowledge base to be relevant to a conversation. Can be used to restrict knowledge bases to certain types of users. A null value will remove the precondition from the knowledge base, it will be available on all conversations.

        segment_id : typing.Optional[EntityId]
            The ID of the segment that must be matched for the knowledge base to be relevant to a conversation.
            A null value will remove the segment from the knowledge base, it will be available on all conversations.

            Segments are replacing inline preconditions - a knowledge base may not have both an inline precondition and a segment.
            Inline precondition support will be removed in a future release.

        refresh_frequency : typing.Optional[KnowledgeBaseRefreshFrequency]
            How often the knowledge base should be refreshed.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}",
            method="PATCH",
            json={
                "appId": app_id,
                "name": name,
                "tags": tags,
                "llmInclusionStatus": llm_inclusion_status,
                "precondition": convert_and_respect_annotation_metadata(
                    object_=precondition, annotation=typing.Optional[Precondition], direction="write"
                ),
                "segmentId": convert_and_respect_annotation_metadata(
                    object_=segment_id, annotation=typing.Optional[EntityId], direction="write"
                ),
                "refreshFrequency": refresh_frequency,
            },
            headers={
                "content-type": "application/merge-patch+json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_knowledge_base_version(
        self,
        knowledge_base_reference_id: str,
        *,
        type: KnowledgeBaseVersionType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseVersion]:
        """
        Create a new knowledge base version.

        If an existing version is in progress, then that version will be finalized in an error state.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to create a version for. All other entity ID fields are inferred from the request.

        type : KnowledgeBaseVersionType
            Indicates whether the completed version constitutes a full or partial refresh of the knowledge base. Deleting and updating documents is only supported for partial refreshes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseVersion]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/version",
            method="POST",
            json={
                "type": type,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseVersion,
                    parse_obj_as(
                        type_=KnowledgeBaseVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def finalize_knowledge_base_version(
        self,
        knowledge_base_reference_id: str,
        *,
        version_id: typing.Optional[EntityIdWithoutAgent] = OMIT,
        status: typing.Optional[KnowledgeBaseVersionFinalizeStatus] = OMIT,
        error_message: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseVersion]:
        """
        Finalize the latest knowledge base version. Required to indicate the version is complete. Will throw an exception if the latest version is not in progress.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to finalize a version for. All other entity ID fields are inferred from the request.

        version_id : typing.Optional[EntityIdWithoutAgent]
            ID that uniquely identifies which knowledge base version to finalize. If not provided will use the most recent version of the knowledge base.

        status : typing.Optional[KnowledgeBaseVersionFinalizeStatus]
            Whether the knowledge base version processing was successful or not.

        error_message : typing.Optional[str]
            A user-facing error message that provides more details about a version failure.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseVersion]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/version/finalize",
            method="POST",
            json={
                "versionId": convert_and_respect_annotation_metadata(
                    object_=version_id, annotation=EntityIdWithoutAgent, direction="write"
                ),
                "status": status,
                "errorMessage": error_message,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseVersion,
                    parse_obj_as(
                        type_=KnowledgeBaseVersion,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_knowledge_base_versions(
        self,
        knowledge_base_reference_id: str,
        *,
        app_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseVersionsListResponse]:
        """
        List all active versions for a knowledge base. Returns the most recent versions first.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to list versions for. All other entity ID fields are inferred from the request.

        app_id : typing.Optional[str]
            The App ID of the knowledge base. If not provided the ID of the calling app will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseVersionsListResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/versions",
            method="GET",
            params={
                "appId": app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseVersionsListResponse,
                    parse_obj_as(
                        type_=KnowledgeBaseVersionsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def search_knowledge_documents(
        self,
        *,
        sort: typing.Optional[KnowledgeDocumentField] = OMIT,
        filter: typing.Optional[KnowledgeDocumentFilter] = OMIT,
        page: typing.Optional[int] = OMIT,
        size: typing.Optional[int] = OMIT,
        sort_desc: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeDocumentsResponse]:
        """
        Search knowledge documents

        Parameters
        ----------
        sort : typing.Optional[KnowledgeDocumentField]

        filter : typing.Optional[KnowledgeDocumentFilter]

        page : typing.Optional[int]
            Page number to return, defaults to 0

        size : typing.Optional[int]
            The size of the page to return, defaults to 20

        sort_desc : typing.Optional[bool]
            Whether to sort descending, defaults to true

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeDocumentsResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/knowledge/documents/search",
            method="POST",
            json={
                "sort": sort,
                "filter": convert_and_respect_annotation_metadata(
                    object_=filter, annotation=KnowledgeDocumentFilter, direction="write"
                ),
                "page": page,
                "size": size,
                "sortDesc": sort_desc,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeDocumentsResponse,
                    parse_obj_as(
                        type_=KnowledgeDocumentsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_knowledge_document(
        self,
        knowledge_base_reference_id: str,
        *,
        knowledge_document_id: EntityIdBase,
        content_type: KnowledgeDocumentContentType,
        content: str,
        title: str,
        version_id: typing.Optional[EntityIdWithoutAgent] = OMIT,
        metadata: typing.Optional[typing.Dict[str, str]] = OMIT,
        url: typing.Optional[str] = OMIT,
        language: typing.Optional[str] = OMIT,
        created_at: typing.Optional[dt.datetime] = OMIT,
        updated_at: typing.Optional[dt.datetime] = OMIT,
        author: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeDocumentResponse]:
        """
        Create or update a knowledge document. Requires an existing knowledge base with an in progress version.
        Will throw an exception if the latest version is not in progress.

        <Tip>
        This API maintains document version history. If for the same reference ID none of the `title`, `text`, `sourceUrl`, `metadata` fields
        have changed, a new document version will not be created. The existing version will be reused.
        </Tip>

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base to create a document for. All other entity ID fields are inferred from the request.

        knowledge_document_id : EntityIdBase
            ID that uniquely identifies this knowledge document within its knowledge base

        content_type : KnowledgeDocumentContentType

        content : str
            The content of the document. Not shown directly to users. May be provided in HTML or markdown. HTML will be converted to markdown automatically. Images are not currently supported and will be ignored.

        title : str
            The title of the document. Will be shown as part of answers.

        version_id : typing.Optional[EntityIdWithoutAgent]
            ID that uniquely identifies which knowledge base version to create the document in. If not provided will use the most recent version of the knowledge base.

        metadata : typing.Optional[typing.Dict[str, str]]
            Metadata for the knowledge document.

        url : typing.Optional[str]
            The URL of the document. Should be visible to end users. Will be shown as part of answers. Not used for crawling.

        language : typing.Optional[str]
            The document language. Must be a valid ISO 639-1 language code.

        created_at : typing.Optional[dt.datetime]
            The time at which this document was created.

        updated_at : typing.Optional[dt.datetime]
            The time at which this document was last modified.

        author : typing.Optional[str]
            The name of the author who created this document.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeDocumentResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/document",
            method="POST",
            json={
                "knowledgeDocumentId": convert_and_respect_annotation_metadata(
                    object_=knowledge_document_id, annotation=EntityIdBase, direction="write"
                ),
                "versionId": convert_and_respect_annotation_metadata(
                    object_=version_id, annotation=EntityIdWithoutAgent, direction="write"
                ),
                "contentType": content_type,
                "content": content,
                "metadata": metadata,
                "title": title,
                "url": url,
                "language": language,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "author": author,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeDocumentResponse,
                    parse_obj_as(
                        type_=KnowledgeDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_knowledge_document(
        self,
        knowledge_base_reference_id: str,
        knowledge_document_reference_id: str,
        *,
        version_id: EntityIdWithoutAgent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Delete knowledge document from a specific version.
        Requires an existing knowledge base with an in progress version of type PARTIAL. Will throw an exception if the version is not in progress.

        Parameters
        ----------
        knowledge_base_reference_id : str
            The reference ID of the knowledge base that contains the document to delete. All other entity ID fields are inferred from the request

        knowledge_document_reference_id : str
            The reference ID of the knowledge document to delete. All other entity ID fields are inferred from the request.

        version_id : EntityIdWithoutAgent
            ID that uniquely identifies which knowledge base version to delete the document from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/{jsonable_encoder(knowledge_base_reference_id)}/{jsonable_encoder(knowledge_document_reference_id)}/document",
            method="DELETE",
            json={
                "versionId": convert_and_respect_annotation_metadata(
                    object_=version_id, annotation=EntityIdWithoutAgent, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_knowledge_document(
        self,
        knowledge_base_version_reference_id: str,
        knowledge_document_reference_id: str,
        *,
        knowledge_base_version_app_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeDocumentResponse]:
        """
        Get a knowledge document by its supplied version and document IDs. Response includes document content in markdown format.

        Parameters
        ----------
        knowledge_base_version_reference_id : str
            The reference ID of the knowledge base version that contains the document. All other entity ID fields are inferred from the request.

        knowledge_document_reference_id : str
            The reference ID of the knowledge document to get. All other entity ID fields are inferred from the request.

        knowledge_base_version_app_id : str
            The App ID of the knowledge base version.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeDocumentResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/knowledge/versions/{jsonable_encoder(knowledge_base_version_reference_id)}/documents/{jsonable_encoder(knowledge_document_reference_id)}",
            method="GET",
            params={
                "knowledgeBaseVersionAppId": knowledge_base_version_app_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeDocumentResponse,
                    parse_obj_as(
                        type_=KnowledgeDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_base import EntityIdBase
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .base_organization import BaseOrganization


class Organization(BaseOrganization):
    organization_id: typing_extensions.Annotated[EntityIdBase, FieldMetadata(alias="organizationId")] = pydantic.Field()
    """
    ID that uniquely identifies this organization
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id import EntityId
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class AgentCapabilityBase(UniversalBaseModel):
    name: str
    description: typing.Optional[str] = None
    created_at: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="createdAt")] = pydantic.Field()
    """
    Timestamp when the capability was created
    """

    enabled: bool = pydantic.Field()
    """
    Whether this capability will be called by Maven.
    """

    version: int
    capability_id: typing_extensions.Annotated[EntityId, FieldMetadata(alias="capabilityId")] = pydantic.Field()
    """
    ID that uniquely identifies this capability
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

LlmPersona = typing.Union[
    typing.Literal[
        "CASUAL_BUDDY",
        "EMPATHETIC_SUPPORTER",
        "FORMAL_PROFESSIONAL",
        "CONCISE_EXPERT",
        "ENTHUSIASTIC_HELPER",
        "PATIENT_EDUCATOR",
        "PIRATE",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawAssetsClient, RawAssetsClient
from .types.initiate_asset_upload_response import InitiateAssetUploadResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AssetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAssetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAssetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAssetsClient
        """
        return self._raw_client

    def initiate_upload(
        self, *, type: str, name: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> InitiateAssetUploadResponse:
        """
        Initiate an upload.
        Returns a pre-signed URL for direct file upload and an asset ID for subsequent operations.

        Parameters
        ----------
        type : str
            The mime-type of the attachment. Supported types are:
            - image/jpeg
            - image/jpg
            - image/png
            - image/gif
            - image/webp
            - application/pdf
            - text/plain
            - text/csv
            - application/vnd.openxmlformats-officedocument.wordprocessingml.document
            - application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
            - application/vnd.openxmlformats-officedocument.presentationml.presentation
            - application/msword
            - application/vnd.ms-excel
            - application/vnd.ms-powerpoint
            - audio/aac
            - audio/mpeg
            - audio/mp4
            - audio/wav
            - audio/ogg
            - video/mp4
            - video/webm

        name : typing.Optional[str]
            An optional name for the attachment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InitiateAssetUploadResponse

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.assets.initiate_upload(
            type="type",
        )
        """
        _response = self._raw_client.initiate_upload(type=type, name=name, request_options=request_options)
        return _response.data

    def commit_upload(
        self,
        asset_reference_id: str,
        *,
        checksum: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Commit an upload after successful file transfer.
        Updates the asset status and makes it available for use.

        Parameters
        ----------
        asset_reference_id : str
            The reference ID of the asset to commit (provided by the initiate call). All other entity ID fields are inferred from the API request.

        checksum : typing.Optional[str]
            Checksum of the uploaded file (optional verification)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mavenagi import MavenAGI

        client = MavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )
        client.assets.commit_upload(
            asset_reference_id="assetReferenceId",
        )
        """
        _response = self._raw_client.commit_upload(
            asset_reference_id, checksum=checksum, request_options=request_options
        )
        return _response.data


class AsyncAssetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAssetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAssetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAssetsClient
        """
        return self._raw_client

    async def initiate_upload(
        self, *, type: str, name: typing.Optional[str] = OMIT, request_options: typing.Optional[RequestOptions] = None
    ) -> InitiateAssetUploadResponse:
        """
        Initiate an upload.
        Returns a pre-signed URL for direct file upload and an asset ID for subsequent operations.

        Parameters
        ----------
        type : str
            The mime-type of the attachment. Supported types are:
            - image/jpeg
            - image/jpg
            - image/png
            - image/gif
            - image/webp
            - application/pdf
            - text/plain
            - text/csv
            - application/vnd.openxmlformats-officedocument.wordprocessingml.document
            - application/vnd.openxmlformats-officedocument.spreadsheetml.sheet
            - application/vnd.openxmlformats-officedocument.presentationml.presentation
            - application/msword
            - application/vnd.ms-excel
            - application/vnd.ms-powerpoint
            - audio/aac
            - audio/mpeg
            - audio/mp4
            - audio/wav
            - audio/ogg
            - video/mp4
            - video/webm

        name : typing.Optional[str]
            An optional name for the attachment.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InitiateAssetUploadResponse

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.assets.initiate_upload(
                type="type",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.initiate_upload(type=type, name=name, request_options=request_options)
        return _response.data

    async def commit_upload(
        self,
        asset_reference_id: str,
        *,
        checksum: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Commit an upload after successful file transfer.
        Updates the asset status and makes it available for use.

        Parameters
        ----------
        asset_reference_id : str
            The reference ID of the asset to commit (provided by the initiate call). All other entity ID fields are inferred from the API request.

        checksum : typing.Optional[str]
            Checksum of the uploaded file (optional verification)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mavenagi import AsyncMavenAGI

        client = AsyncMavenAGI(
            organization_id="YOUR_ORGANIZATION_ID",
            agent_id="YOUR_AGENT_ID",
            app_id="YOUR_APP_ID",
            app_secret="YOUR_APP_SECRET",
        )


        async def main() -> None:
            await client.assets.commit_upload(
                asset_reference_id="assetReferenceId",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.commit_upload(
            asset_reference_id, checksum=checksum, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from .base_attachment import BaseAttachment
from .entity_id_base import EntityIdBase


class AttachmentRequest(BaseAttachment):
    """
    Attachments can be created either with inline data (up to 5MB) using the `content` field or by
    referencing an asynchronously uploaded asset using the `assetId` field.

    See the Assets APIs for more details on how to upload assets and get an asset ID.

    <Tip>
      Due to backwards compatibility, both the `type` field and the `name` field are present when attaching an Asset by ID.
      These fields will be ignored in favor of the values supplied during the initial Asset upload.
      They are only used for inline content uploads.
    </Tip>
    """

    content: typing.Optional[str] = pydantic.Field(default=None)
    """
    Inline attachment data, up to 5MB.
    """

    asset_id: typing_extensions.Annotated[typing.Optional[EntityIdBase], FieldMetadata(alias="assetId")] = (
        pydantic.Field(default=None)
    )
    """
    The ID of an asynchronously uploaded asset.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_filter import EntityIdFilter
from ...commons.types.feedback_type import FeedbackType
from ...commons.types.number_range import NumberRange
from ...commons.types.quality import Quality
from ...commons.types.quality_reason import QualityReason
from ...commons.types.response_length import ResponseLength
from ...commons.types.sentiment import Sentiment
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .resolution_status import ResolutionStatus
from .simulation_filter import SimulationFilter


class ConversationFilter(UniversalBaseModel):
    search: typing.Optional[str] = pydantic.Field(default=None)
    """
    Full-text search query for matching conversations by content. 
    When you search with this parameter, you're performing a full-text search across all textual content 
    in the conversations, including both the user's messages and the AI's responses.
    
    This field also supports a syntax for advanced filtering the `metadata` and `tags` fields.           
    
    Metadata examples:
    - `metadata:myvalue` - matches conversations with any metadata field set to `myvalue`
    - `metadata.mykey:myvalue` - matches conversations with a metadata field `mykey` set to `myvalue`
    - `metadata.mykey:myvalue OR anothervalue` - matches conversations with a metadata field `mykey` set to `myvalue` or `anothervalue`
    - `metadata.mykey:*` - matches conversations with a metadata field `mykey`
    - `-metadata:myvalue` - matches conversations that do not have any metadata field set to `myvalue`
    - `_exists_:metadata` - matches conversations that have any metadata field set
    
    Tags examples:
    - `tags:myvalue` - matches conversations with a tag of `myvalue`
    - `tags:myvalue OR anothervalue` - matches conversations with a tag of `myvalue` or `anothervalue`
    - `-tags:myvalue` - matches conversations that do not have the tag `myvalue`
    - `_exists_:tags` - matches conversations that have any tags field set
    """

    created_after: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdAfter")] = (
        pydantic.Field(default=None)
    )
    """
    Filter conversations created on or after this timestamp
    """

    created_before: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="createdBefore")] = (
        pydantic.Field(default=None)
    )
    """
    Filter conversations created on or before this timestamp
    """

    apps: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter by app IDs
    """

    categories: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter by conversation categories
    """

    actions: typing.Optional[typing.List[EntityIdFilter]] = pydantic.Field(default=None)
    """
    Filter by actions that were executed in the conversation
    """

    incomplete_actions: typing_extensions.Annotated[
        typing.Optional[typing.List[EntityIdFilter]], FieldMetadata(alias="incompleteActions")
    ] = pydantic.Field(default=None)
    """
    Filter by actions that were suggested but not completed by the AI agent
    """

    feedback: typing.Optional[typing.List[FeedbackType]] = pydantic.Field(default=None)
    """
    Filter by feedback types received in the conversation. 
    This is a legacy field that maps to Events saved in the system for `ThumbsUp`, `ThumbsDown`, and `Insert`. 
    The `Handoff` filter will pass if any bot responses on the conversation returned the system fallback message; there are no corresponding handoff events.
    """

    human_agents: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="humanAgents")] = (
        pydantic.Field(default=None)
    )
    """
    Filter by human agents who participated in the conversation
    """

    human_agents_with_inserts: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="humanAgentsWithInserts")
    ] = pydantic.Field(default=None)
    """
    Filter by human agents who inserted a maven AI generated suggestion in the conversation
    """

    languages: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter by conversation languages
    """

    quality: typing.Optional[typing.List[Quality]] = pydantic.Field(default=None)
    """
    Filter by AI assessed conversation quality classification
    """

    quality_reason: typing_extensions.Annotated[
        typing.Optional[typing.List[QualityReason]], FieldMetadata(alias="qualityReason")
    ] = pydantic.Field(default=None)
    """
    Filter by AI assessed quality reason classification
    """

    response_length: typing_extensions.Annotated[
        typing.Optional[typing.List[ResponseLength]], FieldMetadata(alias="responseLength")
    ] = pydantic.Field(default=None)
    """
    Filter by AI response length classification
    """

    sentiment: typing.Optional[typing.List[Sentiment]] = pydantic.Field(default=None)
    """
    Filter by AI assessed sentiment analysis
    """

    tags: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Filter by tags applied to the conversation
    """

    agent_user_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[str]], FieldMetadata(alias="agentUserIds")
    ] = pydantic.Field(default=None)
    """
    Filter by agent user IDs associated with the conversation
    """

    resolution_status: typing_extensions.Annotated[
        typing.Optional[typing.List[ResolutionStatus]], FieldMetadata(alias="resolutionStatus")
    ] = pydantic.Field(default=None)
    """
    Filter by conversation resolution status which is determined by AI based on the conversation content.
    """

    resolved_by_maven: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="resolvedByMaven")] = (
        pydantic.Field(default=None)
    )
    """
    Filter conversations based on whether they were resolved by Maven AI
    """

    user_message_count: typing_extensions.Annotated[
        typing.Optional[NumberRange], FieldMetadata(alias="userMessageCount")
    ] = pydantic.Field(default=None)
    """
    Filter by the number of messages sent by the user in the conversation
    """

    has_attachment: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="hasAttachment")] = (
        pydantic.Field(default=None)
    )
    """
    Filter by whether any message in the conversation has an attachment
    """

    matched_segment_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[EntityIdFilter]], FieldMetadata(alias="matchedSegmentIds")
    ] = pydantic.Field(default=None)
    """
    Filter by the segments that any message on a conversation matched.
    """

    inbox_item_ids: typing_extensions.Annotated[
        typing.Optional[typing.List[EntityIdFilter]], FieldMetadata(alias="inboxItemIds")
    ] = pydantic.Field(default=None)
    """
    Filter by inbox item IDs associated with the conversation
    """

    simulation_filter: typing_extensions.Annotated[
        typing.Optional[SimulationFilter], FieldMetadata(alias="simulationFilter")
    ] = pydantic.Field(default=None)
    """
    Whether to include simulation conversations in search results. Defaults to only non-simulation conversations.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .base_organization import BaseOrganization
from .create_organization_request import CreateOrganizationRequest
from .organization import Organization
from .organization_patch_request import OrganizationPatchRequest

__all__ = ["BaseOrganization", "CreateOrganizationRequest", "Organization", "OrganizationPatchRequest"]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..commons.errors.bad_request_error import BadRequestError
from ..commons.errors.not_found_error import NotFoundError
from ..commons.errors.server_error import ServerError
from ..commons.types.error_message import ErrorMessage
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from .types.search_app_settings_response import SearchAppSettingsResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAppSettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search(
        self, *, index: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SearchAppSettingsResponse]:
        """
        Search for app settings which have the `$index` key set to the provided value.

        You can set the `$index` key using the Update app settings API.

        <Warning>This API currently requires an organization ID and agent ID for any agent which is installed on the app. This requirement will be removed in a future update.</Warning>

        Parameters
        ----------
        index : str
            Will return all settings which have the `$index` key set to the provided value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchAppSettingsResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/app-settings/search",
            method="GET",
            params={
                "index": index,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchAppSettingsResponse,
                    parse_obj_as(
                        type_=SearchAppSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Get app settings set during installation

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/app-settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        request: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Update app settings. Performs a merge of the provided settings with the existing app settings.

        - If a new key is provided, it will be added to the app settings.
        - If an existing key is provided, it will be updated.
        - No keys will be removed.

        Note that if an array value is provided it will fully replace an existing value as arrays cannot be merged.

        Parameters
        ----------
        request : typing.Dict[str, typing.Optional[typing.Any]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/app-settings",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAppSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search(
        self, *, index: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SearchAppSettingsResponse]:
        """
        Search for app settings which have the `$index` key set to the provided value.

        You can set the `$index` key using the Update app settings API.

        <Warning>This API currently requires an organization ID and agent ID for any agent which is installed on the app. This requirement will be removed in a future update.</Warning>

        Parameters
        ----------
        index : str
            Will return all settings which have the `$index` key set to the provided value.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchAppSettingsResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/app-settings/search",
            method="GET",
            params={
                "index": index,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchAppSettingsResponse,
                    parse_obj_as(
                        type_=SearchAppSettingsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Get app settings set during installation

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/app-settings",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        request: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]:
        """
        Update app settings. Performs a merge of the provided settings with the existing app settings.

        - If a new key is provided, it will be added to the app settings.
        - If an existing key is provided, it will be updated.
        - No keys will be removed.

        Note that if an array value is provided it will fully replace an existing value as arrays cannot be merged.

        Parameters
        ----------
        request : typing.Dict[str, typing.Optional[typing.Any]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Optional[typing.Any]]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/app-settings",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise ServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorMessage,
                        parse_obj_as(
                            type_=ErrorMessage,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

Capability = typing.Union[
    typing.Literal["MARKDOWN", "FORMS", "IMAGES", "CHARTS_HIGHCHARTS_TS", "ASYNC", "OAUTH_BUTTONS"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...commons.types.entity_id_without_agent import EntityIdWithoutAgent
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .conversation_message_request import ConversationMessageRequest


class DeliverMessageRequest_User(UniversalBaseModel):
    type: typing.Literal["user"] = "user"
    user_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="userId")]
    message: ConversationMessageRequest

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class DeliverMessageRequest_Conversation(UniversalBaseModel):
    type: typing.Literal["conversation"] = "conversation"
    conversation_id: typing_extensions.Annotated[EntityIdWithoutAgent, FieldMetadata(alias="conversationId")]
    message: ConversationMessageRequest

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


DeliverMessageRequest = typing.Union[DeliverMessageRequest_User, DeliverMessageRequest_Conversation]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .event_trigger_base import EventTriggerBase
from .event_trigger_request import EventTriggerRequest
from .event_trigger_response import EventTriggerResponse
from .event_trigger_type import EventTriggerType
from .event_triggers_search_request import EventTriggersSearchRequest
from .event_triggers_search_response import EventTriggersSearchResponse
from .trigger_field import TriggerField
from .trigger_partial_update import TriggerPartialUpdate

__all__ = [
    "EventTriggerBase",
    "EventTriggerRequest",
    "EventTriggerResponse",
    "EventTriggerType",
    "EventTriggersSearchRequest",
    "EventTriggersSearchResponse",
    "TriggerField",
    "TriggerPartialUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import typing

TriggerType = typing.Union[
    typing.Literal["CONVERSATION_CREATED", "FEEDBACK_CREATED", "INBOX_ITEM_CREATED", "EVENT_CREATED"], typing.Any
]
